#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <log_it.h>
#include <utils_crypt.h>

#define CRYPT
#define LOGBUFFER 1024

int log_it(const char *file, const char *format, ...)
{
   FILE *log = 0;
   va_list ap;
   va_start(ap, format);

   if(!file) {
	   fprintf(stderr, "[ERROR] No logfile!\n");
	   return -1;
   }
   if((log = fopen(file, "a")) == NULL) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", file);
	   perror("[ERROR] log_it(): ");
	   return -2;
   }
   if(vfprintf(log, format, ap) < 0) {
	   fprintf(stderr, "[ERROR] in writing to %s\n", file);
   }
   va_end(ap);
   
   if(fclose(log) != 0) {
	   fprintf(stderr, "[ERROR] in closing %s\n", file);
   }

   return 0;
}

#ifdef CRYPT
int crypt_log_it(const char *file, const char *key, const char *format, ...)
{
   FILE *log = 0;
   va_list ap;
   char *ctext;
   char buffer[LOGBUFFER + 1] = { '\0' };

   va_start(ap, format);

   if(!file) {
	   fprintf(stderr, "[ERROR] No logfile!\n");
	   return -1;
   }
   if(!key) {
	   fprintf(stderr, "[ERROR] No key logfile!\n");
	   return -2;
   }
   if((log = fopen(file, "a")) == NULL) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", file);
	   perror("[ERROR] log_it(): ");
	   return -3;
   }
   
   vsnprintf(buffer, LOGBUFFER, format, ap);
   ctext = crypt_it(key, buffer);

   if(fprintf(log, "%s\n", ctext) < 0) {
	   fprintf(stderr, "[ERROR] in writing to %s\n", file);
   }
   free(ctext);
   va_end(ap);

   if(fclose(log) != 0) {
	   fprintf(stderr, "[ERROR] in closing %s\n", file);
   }
   
   return 0;
}
#endif
