#include <stdio.h>
#include <errno.h>
#include <stdbool.h>
#include <string.h>
#include <stdlib.h>

#include <logs_text1.h>
#include <utils_replace.h>
#include <log_it.h>

/*
 * Serach and in replace in file, and output to to_file:
 * It makes more or less "sed s/string/replace/g file >to_file"
 *
 * Return:
 *     On sucess: Number of changes 
 *     Error:
 *     		-1	No file 
 *     		-2	No target file
 *     		-3	No string to search
 *     		-4	No string for replace
 *     		-5	Error in opening file
 *     		-6	Error in opening target file
 *     		-7	Error in opening logfile
 *     		-8	Error in reading file
 *     		-9	Error in writing to target file
 *     		-10	Buffer error (Rest is to short)
 *     		-11	Error in writing the rest into file
 *     		
 *     		
 *     		
 */
int replace_in_file(const char *file, 
                    const char *to_file,
		              const char *string,
		              const char *replace,
		              int verbose, const char *logfile, const char *key)
{
   FILE *f_read = 0, *f_write = 0, *f_log = 0;
   int data_to_read = 0, data_read = 0;
   int len_clean = 0, len_rest = 0, end = REST;
   int count = 0, changed = 0, changed_tmp = 0;
   char *clean_buffer, *read_buffer, *rest_buffer, *p_buffer;
   bool is_clean = true;
   char *buffer = 0;
   
   /* Idiot checking (newer trust programmers!) */
   if(!file) {
	   fprintf(stderr, "[ERROR] No file to clean!\n");
	   return -1;
   }
   if(!to_file) {
	   fprintf(stderr, "[ERROR] No file to write to!\n");
	   return -2;
   }
   if(!string) {
	   fprintf(stderr, "[ERROR] No string for cleaning!\n");
	   return -3;
   }
   if(!replace) {
	   /*return -4; */
      /* set a default string to replace and go on...*/
      replace=DEFAULT_REPLACE;
   }
   if((buffer = (char *)malloc(BUFFER + REST + 1)) == 0) {
	   fprintf(stderr, "[ERROR]  replace_text: malloc()\n");
	   return 0;
   }
   memset(buffer, 0x0, BUFFER + REST + 1);
   
   /* give status message */
   if(verbose > 2) {
	   printf("[STATUS]	s/%s/%s/g in %s\n", string, replace, file);
   }
   
   /* Open all files */
   if((f_read = fopen(file, "r")) == 0) {
      fprintf(stderr, "[ERROR] in read opening %s\n", file);
	   perror("[ERROR] replace_in_file(): ");
	   free(buffer);
	   return -5;
   }
   if((f_write = fopen(to_file, "w")) == 0) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", to_file);
	   perror("[ERROR] replace_in_file(): ");
	   free(buffer);
	   return -6;
   }
   if(logfile) {
	   if((f_log = fopen(logfile, "w")) == 0) {
	      fprintf(stderr, "[ERROR] in write opening %s\n", logfile);
	      perror("[ERROR] replace_in_file(): ");
	      free(buffer);
	      return -7;
	   }
   }

   /* ok let's start.... */
   data_to_read = BUFFER;
   read_buffer = buffer;
   
   /* start reading file */
   while(!feof(f_read)) {
	   memset(read_buffer, 0, BUFFER);
	   is_clean = true;
	   changed_tmp = 0;
      
	   /* if we read the first time, readbuffer is bigger */
	   if(count < 1) {
         data_to_read = BUFFER;
	      read_buffer = buffer;
	   }
	   count++;

	   /* read */
	   data_read = fread(read_buffer, 1, data_to_read, f_read);
	   if(data_read < 1) {
         fprintf(stderr, "[ERROR] fread() %s\n", file);
	      free(buffer);
	      return -8;
	   }

	   /* now clear the buffer */
	   /* clean_buffer is the cleaned buffer, which does NOT contain string */
	   clean_buffer = do_replace_text_plain(buffer, string, replace, &changed_tmp);
	   changed = changed + changed_tmp;
	   len_clean = strlen(clean_buffer);

	   /* write to file */
	   if(fwrite(clean_buffer, 1, len_clean - REST, f_write) != len_clean - REST) {
         fprintf(stderr, "[ERROR] fwrite() %s\n", to_file);
	      free(buffer);
	      if(clean_buffer != buffer) {
		      free(clean_buffer);
	      }
	      return -9;
	   }
      if(changed_tmp > 0) {	/* buffer IS changed */
         p_buffer = buffer;
	      p_buffer = strtok(p_buffer, "\n");
	      while(p_buffer) {
		      if(strstr(p_buffer, string) != 0) {
               /* verbose */
		         if(verbose > 3) {
			         printf("FOUND: %s\n", p_buffer);
		         }
		         /* log it */
		         if(logfile) {
			         if(key) {
			            crypt_log_it(logfile, key, "FOUND: %s\n", p_buffer);
			         } else {
			            log_it(logfile, "FOUND: %s\n", p_buffer);
			         }
		         }
		      }
		      p_buffer = strtok(0, "\n");
	      }
      }
      len_rest = len_clean - REST;
	   if(len_rest < 1) {
         fprintf(stderr, "[ERROR] replace_in_file()\n");
	      free(buffer);
	      if(clean_buffer != buffer) {
		      free(clean_buffer);
	      }
	      return -10;
	   }
	
      /* the last 256 */
	   rest_buffer = clean_buffer + len_rest;
	
      /* copy the rest to the beginning */
	   memmove(buffer, rest_buffer, REST);
	
      /* jump */
	   read_buffer = buffer + REST;

	   if(clean_buffer != buffer) {
         free(clean_buffer);
	   }
   } /* EOF */

   /* now we have to write the rest */
      len_rest = len_clean - REST;
      if(len_rest < REST) {
	      end = len_rest;
      } else {
	   end = REST;
      }
      rest_buffer = clean_buffer + len_rest;
      
      if(fwrite(rest_buffer, 1, end, f_write) != end) { 
	      fprintf(stderr, "[ERROR] in writing reststring to %s\n", to_file);
	      free(buffer);
	      if(clean_buffer != buffer) {
	         free(clean_buffer);
	      }
	      return -11;
      }
      memset(buffer, 0, BUFFER + REST + 1);

      if(f_log != 0) {
	      fclose(f_log);
      }
      fclose(f_read);
      fclose(f_write);

      if(verbose > 2) {
	      printf("[STATUS]	changed = %d\n", changed);
      }
      if(!clean_buffer)
         free(clean_buffer);
      if(!free) 
         free(buffer);

      return(changed);
}
