#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <utils_replace.h>


/*
 * easy search and replace
 *
 * 	Return:
 * 		returns changed string
 * 		or NULL if it did not find the search-string
 * 		or NULL is there was an error
 * 		
 */
char *replace_text_plain(char *source, const char *string, const char *replace)
{
   size_t len_source = 0;
   size_t len_string = 0;
   size_t len_replace = 0;
   size_t len_buffer = 0;
   size_t len_match = 0;
   size_t len_tmp = 0;
   char *buffer = 0, *p_buffer;
   char *match = 0;

   if(!source) {
	   fprintf(stderr, "[ERROR] no source string for replace_text()!\n");
	   return 0;
   }
   if(!string) {
	   fprintf(stderr, "[ERROR] no string for replace_text()!\n");
	   return 0;
   }
   if(!replace) {
	   fprintf(stderr, "[ERROR] no replace string for replace_text()!\n");
	   return 0;
   }

   len_source = strlen(source);
   len_string = strlen(string);
   len_replace = strlen(replace);
   len_buffer = len_source - len_string + len_replace + 4;

   /*  grep */
   match = strstr(source, string);
   if(match == 0) {
	   return (0);
   }
   if((buffer = (char *) malloc(len_buffer)) == 0) {
	   fprintf(stderr, "[ERROR]  replace_text: malloc()\n");
	   return 0;
   }
   memset(buffer, 0, len_buffer);
   p_buffer = buffer;
   len_match = strlen(match);
   len_tmp = len_source - len_match;
   
   memcpy(p_buffer, source, len_tmp);
   p_buffer = p_buffer + len_tmp;
   
   memcpy(p_buffer, replace, len_replace);
   p_buffer = p_buffer + len_replace;

   match = match + len_string;
   len_match = strlen(match);
   memcpy(p_buffer, match, len_match);

   return(buffer);
}

/*
 * easy search and replace ALL matching substrings in a given string.
 *   *counter is the number of changes in the given string
 *
 *   Return: 
 *   		changed string
 *   		or NULL on error
 */
char *do_replace_text_plain(char *source, const char *string, 
                            const char *replace, int *counter)
{
   int count = 0;
   char *dest = 0;
   char *newstr = 0, *newstr_t = 0;
   char *grep;

   dest = 0;

   /* Idiot checking */
   if(!source) {
	   fprintf(stderr, "[ERROR] No source for replace\n");
	   *counter = -1;
	   return 0;
   }
   if(!string) {
	   fprintf(stderr, "[ERROR] No string for replace\n");
	   *counter = -2;
	   return 0;
   }
   if(!replace) {
      fprintf(stderr, "[ERROR] No replace_string to replace\n");
	   *counter = -3;
	   return 0;
   }

   /* grep */
   grep = strstr(source, string);
   if(grep != 0) { /* string found */
	   newstr = replace_text_plain(source, string, replace);
	if(newstr) {
      count++;
	   /* now change ALL matches */
	   while((newstr_t = replace_text_plain(newstr, string, replace)) != 0) {
		   free(newstr);
		   newstr = newstr_t;
		   count++;
	   }

	   /* set destination to a clean buffer */
	   dest = newstr;
   } else {
      fprintf(stderr, "[ERROR] \n");
	   *counter = -4;
	   return 0;
	}
   } else {	/* string not found */
	   /* source IS clean, so it is a clean buffer */
	   dest = source;
   }
   *counter = count;
   
   return(dest);
}
