#include <stdio.h>
#include <string.h>
#include <utime.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

/* own */
#include <logcl.h>
#include <misc_add.h>
#include <default_syslog.h>
#include <defaults_syslog.h>

/*
 * reads syslog_config to check for logfiles. 
 * All logfiles from the config file are added to the list of 
 * logfiles to clean.
 * It also checks for remote logging, logging to users, ttys, 
 * pipes, etc, AND gives a warning.
 * 
 */
int read_syslog(block_head_t * mylog,
                struct Cleanme *MyLog, char *file, int type)
{
   FILE *conf;
   char line[BUFF_LONG + 1] = { "\0" };
   char buffer[BUFF_LONG + 1] = { "\0" };
   char logfile[BUFF_LONG + 1] = { "\0" };
   int i, j;
   struct stat perm;
   /*struct utimbuf buf; */
   int verbose = MyLog->verbose;

   /* idiot checking */
   if((type < 0) || (type > 6)) {
	   fprintf(stderr,"[ERROR] don't know how to read logfiles from %s because\
              type %d is unknown", file, type);
	   return -1;
   }
   
   /* idiot file checking I */
   if(stat(file, &perm) != 0) {
      return -2;
   }
   
   /* file exits? */
   /* idiot file checking II */
   if((!S_ISREG(perm.st_mode)) || (perm.st_size < 0)) {
	   fprintf(stderr, "[ERROR] not a regular file %s\n", file);
	   return -3;
   }

   /* save file permissions */
   /*  buf.actime = perm.st_atime;
    buf.modtime = perm.st_mtime; */

   /* read config file */
   if((conf = fopen(file, "r")) == 0) {
	   fprintf(stderr, "[ERROR] in opening %s \n", file);
	   return -4;
   }
   if(verbose > 3) {
	   printf("[SYSLOG_CONF]\t %s checking config, and cleaning\
             these logfiles (%d)\n", file, type);
   }
   
   while(!feof(conf)) {
      memset(buffer, 0, sizeof(buffer));
	   memset(line, 0, sizeof(line));
	   memset(logfile, 0, sizeof(logfile));
	   fgets(buffer, sizeof(buffer), conf);
	   
      /* filter out comments */
	   for (i = 0; i < BUFF_LONG; i++) {
         if (buffer[i] == '#') {
		      break;
         }
	      line[i] = buffer[i];
	   }
	   
      /* old syslog */
	   if(type == 0) {
         if (verbose < 2) {
		      printf("[INFO]\t reading syslog.conf from %s\n", file);
	      }
	      j = 0;
	      for(i = 0; i < strlen(line); i++) {
		      if(line[i] == '@') {
		         printf("\a\n[WARN] remote loghost:   %s", line);
		      }
		      if(line[i] == '/') {
		         while(!isspace(line[i])) {
			         logfile[j] = line[i];
			         j++;
			         i++;
		         }
		      }
	      }
	      if (strlen(logfile) >0) {
            memset(MyLog->name, 0, BUFF_LONG);
	         strncpy(MyLog->name, logfile, BUFF_LONG - 1);
	         addLog(mylog, MyLog);
         }
	   }
	   /* end  old syslog */

	   /* syslog-ng */
	   if (type == 1) {
         if (verbose < 2) {
		      printf("[INFO]\t reading syslog.conf from %s\n", file);
	      }	      
         /* remote loghost checking */
	      if(strstr(line, "udp") != 0) {
		      printf("\a\n[WARN] remote loghost(UDP): %s", line);
	      }
	      if(strstr(line, "tcp") != 0) {
		      printf("\a\n[WARN] remote loghost(TCP): %s", line);
	      }
	      if(strstr(line, "usertty") != 0) {
		      printf("\a\n[WARN] logging to user tty: %s", line);
	      }
	      if(strstr(line, "pipe") != 0) {
		      printf("\a\n[WARN] logging to a pipe: %s", line);
	      }
	      if(strstr(line, "file") != 0) {
		      j = 0;
		      for(i = 0; i < strlen(line); i++) {
		         if(line[i] == '"') {
			         i++;
			            while(line[i] != '"') {
			               logfile[j] = line[i];
			               j++;
			               i++;
			            }
		         }
		      }
	         if (strlen(logfile) >0) {
		         memset(MyLog->name, 0, BUFF_LONG);
		         strncpy(MyLog->name, logfile, BUFF_LONG - 1);
		         addLog(mylog, MyLog);
            }
         }
	   }
	   /* syslog-ng end */

	/* newsyslog */
	if(type == 2) {
      if(verbose < 2) {
		   printf("[INFO]\t reading syslog.conf from %s\n", file);
	   }
	   j = 0;
	   for(i = 0; i < strlen(line); i++) {
		   /* no remote logging? */
		   if(line[i] == '/') {
		      while(!isspace(line[i])) {
			      logfile[j] = line[i];
			      j++;
			      i++;
		      }
		      break;
		   }
	   }
	   if (strlen(logfile) >0) {
	      memset(MyLog->name, 0, BUFF_LONG);
	      strncpy(MyLog->name, logfile, BUFF_LONG - 1);
	      addLog(mylog, MyLog);
      }
	}
	/* end  newsyslog */

	/* msyslog syslog */
	if(type == 3) {
      if(verbose < 2) {
		   printf("[INFO]\t reading syslog.conf from %s\n", file);
	   }
	   if(strstr(line, "%mysql") != 0) {
		   printf("\a\n[WARN] logging to mysql: %s[WARN] you have to remove\
                these logs by hand, but here you have username and password\
                for mysql ;-)\n", line);
	   }
	   if(strstr(line, "%mysql") != 0) {
		   printf("\a\n[WARN] logging to mysql: %s[WARN] you have to remove\
                these logs by hand, but here you have username and password\
                for mysql ;-)\n", line);
	   }
	   /* TOD0: recalulate hashes */
	   if(strstr(line, "%peo") != 0) {
		   printf("\a\n[WARN] hashed logging: %s [WARN] now you have a problem\
                :-(\n", line);
	   }
      j = 0;
      for(i = 0; i < strlen(line); i++) {
		   if(line[i] == '@') {
            printf("\a\n[WARN] remote loghost:   %s\n", line);
		   }
		   if(line[i] == '/') {
		      while(!isspace(line[i])) {
			      logfile[j] = line[i];
			      j++;
			      i++;
		      }
		   }
	   }
	   if (strlen(logfile) >0) {
	      memset(MyLog->name, 0, BUFF_LONG);
	      strncpy(MyLog->name, logfile, BUFF_LONG - 1);
	      addLog(mylog, MyLog);
      }
	}
	/* end msyslog syslog */

	/* sdscsyslogd */
	if(type == 4) {
      if(verbose < 2) {
         printf("[INFO]\t reading syslog.conf from %s\n", file);
	   }
	   /* remote loghost checking */
	   if(strstr(line, "type udp") != 0) {
		   printf("\a\n[WARN] remote loghost(UDP): %s", line);
	   }
	   if(strstr(line, "type beep") != 0) {
		   printf("\a\n[WARN] beep channel logging: %s", line);
	   }
	   if(strstr(line, "type user") != 0) {
		   printf("\a\n[WARN] logging to user tty: %s", line);
	   }
      if(strstr(line, "output-path") != 0) {
		   j = 0;
		   for(i = 0; i < strlen(line); i++) {
		      if((line[i] == '"') && (line[i + 1] == '/')) {
			      i++;
			      while(line[i] != '"') {
			         logfile[j] = line[i];
			         j++;
			         i++;
			      }
		      }
		   }
	      if (strlen(logfile) >0) {
		      memset(MyLog->name, 0, BUFF_LONG);
		      strncpy(MyLog->name, logfile, BUFF_LONG - 1);
		      addLog(mylog, MyLog);
         }
	   }
	}
	/* end sdscsyslogd */

	/* metalog */
	if(type == 5) {
      if(verbose < 2) {
		   printf("[INFO]\t reading syslog.conf from %s\n", file);
	   }
	   /* remote loghost checking */
	   if(strstr(line, "command") != 0) {
		   printf("\a\n[WARN] logging to a command: %s", line);
	   }
	   if(strstr(line, "logdir") != 0) {
		   j = 0;
		   for(i = 0; i < strlen(line); i++) {
            if(line[i] == '"') {
			      i++;
			      while(line[i] != '"') {
			         logfile[j] = line[i];
			         j++;
			         i++;
			      }
		      }
		   }
	      if (strlen(logfile) >0) {
		      memset(MyLog->name, 0, BUFF_LONG);
		      strncpy(MyLog->name, logfile, BUFF_LONG - 1);
		      addLog(mylog, MyLog);
         }
	   }
	}
	/* metalog end */

	/*end */
 }

   /* write at_time and m_time  back again */
   /* set_time(file, perm); */
   fclose(conf);

   return (0);
}

void add_syslogs(block_head_t * mylog, struct Cleanme *MyLog)
{
    int a;
    
    if (MyLog->verbose > 3)
	   printf("[VERBOSE] Scanning default syslog config files\n");

      /* syslog */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for old school syslog\n");
      }
      for(a = 0; syslog_conf_syslog[a] != 0; a++) {
	      if(MyLog->verbose > 5) {
	         printf(" %s, ", syslog_conf_syslog[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_syslog[a], 0);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }
      
      /* syslog-ng */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for syslog-ng\n");
      }
      for(a = 0; syslog_conf_syslog_ng[a] != 0; a++) {
	      if(MyLog->verbose > 5) {
	         printf(" %s, ", syslog_conf_syslog_ng[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_syslog_ng[a], 1);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }
      
      /* newsyslog */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for newsyslog\n");
      }
      for(a = 0; syslog_conf_newsyslog[a] != 0; a++) {
	      if(MyLog->verbose > 5) {
	         printf(" %s, ", syslog_conf_newsyslog[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_syslog[a], 2);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }
      
      /* mysyslog */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for msyslog\n");
      }
      for(a = 0; syslog_conf_mysyslog[a] != 0; a++) {
	      if(MyLog->verbose > 5) {    
	         printf(" %s, ", syslog_conf_mysyslog[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_mysyslog[a], 3);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }  
      
      /* sdscsyslogd */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for sdscsyslogd\n");
      }
      for (a = 0; syslog_conf_sdscsyslogd[a] != 0; a++) {
	      if (MyLog->verbose > 5) {
	         printf(" %s, ", syslog_conf_sdscsyslogd[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_sdscsyslogd[a], 4);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }
      
      /* metalog */
      if(MyLog->verbose > 4) {
	      printf("[VERBOSE] Scanning for metalog\n");
      }   
      for(a = 0; syslog_conf_metalog[a] != 0; a++) {
	      if(MyLog->verbose > 5) {
	         printf(" %s, ", syslog_conf_metalog[a]);
         }
	      read_syslog(mylog, MyLog, syslog_conf_metalog[a], 3);
      }
      if(MyLog->verbose > 5) {
	      printf(" \n");
      }
      
      return;
}
