#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

/* own */
#include <logcl.h>
#include <make_clear.h>
/*#include "../Liblogclean/include/logclean.h" */
#include <logclean.h>

/*
 * This is more or less a wrapper function for the 
 * liblogclean functions.
 *
 */
int clear_log(struct Cleanme *MyLog, char *tmpfile)
{
   char *file = MyLog->name;
   char *searchstring = MyLog->searchstring;
   char *replacestring = MyLog->replacestring;
   char *user = MyLog->user;
   char *host = MyLog->host;
   char *tty = MyLog->tty;
   char *cmd = MyLog->cmd;
   char *logto = MyLog->logto;
   char *crypt = MyLog->crypt;
   int logtype = MyLog->logtype;
   int verbose = MyLog->verbose;
   int is_clean = MyLog->is_clean;
   int checked = MyLog->checked;
   int ignore = MyLog->ignore;
   int clear_core = MyLog->clear_core;
   time_t change_time = MyLog->change_time;
   struct stat perm;
   
   /* checking */
   if(checked == 1) {
	   fprintf(stderr, "[ERROR] file %s is already checked! Ignore and go on\n", 
              file);
   	return -1;
   }
   if(ignore == 1) {
	   fprintf(stderr, "[IGNORE] Ignore %s and go on\n", file);
	   return -3;
   }
   if(stat(file, &perm) != 0) {	/* file exits? */
	   if(MyLog->verbose > 2) {
	      printf("[error] no stat for %s\n", file);
	   }
	   return -3;
   }
   if(strlen(searchstring) == 0) {
	   searchstring = 0;
   }
   if(strlen(replacestring) == 0) {
	   replacestring = 0;
   }
   if(strlen(user) == 0) {
	   user = 0;
   }
   if(strlen(host) == 0) {
	   host = 0;
   }
   if(strlen(tty) == 0) {
	   tty = 0;
   }
   if(strlen(cmd) == 0) {
	   cmd = 0;
   }
   if(strlen(logto) == 0) {
	   logto = 0;
   }
   if(strlen(crypt) == 0) {
	   crypt = 0;
   }
   if (MyLog->compressed != 0){
      if(verbose < 4) {
         printf("[STATUS] Ignoring compressed [%d] file:  %s \n", MyLog->compressed, file);
      }
      return -5;
   }
   if(verbose < 4) {
	   printf("[STATUS] clearing %s \n", file);
   }
   
   switch (logtype) {
    case 1:
	   is_clean = replace_in_file(file, tmpfile, searchstring, replacestring,
                                 verbose, logto, crypt);
	   break;
    case 2:
      is_clean = remove_logs(file, tmpfile, searchstring, verbose, logto, 
                             crypt);
	   break;
    case 3:
      is_clean = wtmpLogs(file, tmpfile, searchstring, replacestring, user,
                          host, verbose, logto, crypt);
	   break;
    case 4: is_clean = wtmpLogs(file, tmpfile, searchstring, replacestring, user,
                                host, verbose, logto, crypt);
	   break;
    case 5:
      is_clean = clearLastlog(file, tmpfile, searchstring, replacestring, user,
                              change_time, verbose, logto, crypt);
	   break;
    case 6:
      is_clean = clearAcct(file, tmpfile, replacestring, cmd, user, tty,
                           clear_core, verbose, logto, crypt);
	   break;
    case 7:
      is_clean = remove_lines(file, tmpfile, searchstring, verbose, logto,
                             crypt);
    case 8:
      is_clean = remove_logs_mod_sec(file, tmpfile, searchstring, verbose, logto,
                               crypt);
      break;
    case 9:
      is_clean = remove_logs_prelude_logs(file, tmpfile, searchstring, verbose, logto,
                               crypt);
      break;
    case 10:
      is_clean = remove_logs_prelude_xml(file, tmpfile, searchstring, verbose, logto,
                               crypt);
      break;
    case 11:
      is_clean = remove_logs_samba(file, tmpfile, searchstring, verbose, logto,
                               crypt);
      break;
   case 12:
      is_clean = remove_logs_snort_alert(file, tmpfile, searchstring, verbose, logto,
                               crypt);
      break;
   case 13:
      is_clean = remove_manual(file, tmpfile, verbose);
      break;   
   
   
    default:
	   fprintf(stderr,"[ERROR] don't know this logtype [%d]! Ignore and go on\n",
              logtype);
	   return -2;
   }
   
   writeBack(tmpfile, file, verbose);
   MyLog->is_clean = is_clean;
   
   if(is_clean > 0) {
	   MyLog->was_clean = 0;
   }
   set_time(file, &perm);

   return(is_clean);
}


int remove_manual(const char *file, const char *to_file, int verbose)
{
   int len;
   char tmp[6];
   printf(" Remove it manual\n");
   writeBack(file, to_file, verbose);
   printf("OK, now you can edit %s\n",to_file);
   printf("When you are ready enter \"done\": ");
   for(;;){
      fgets(tmp, 6, stdin);
      len = strlen(tmp);
      if(len > 1) {
         if(strcmp(tmp, "done\n") == 0) {
            /* in logclean-ng make_clear() writes back, so I just return here */
            return 1;
         }
         else{
            printf("\nIdiot!\n Enter done: ");
         }
      } else{
         printf("\nIDIOT\n Enter done: ");
      }
   }
   return -1;
}
