#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>


/* own */
#include <logcl.h>
#include <misc_listen.h>
#include <misc_add.h>


/*
 * check bevor adding to the list
 * check for file (already in the list, exist,file,perms)
 */
int checkLog(block_head_t *head, char *name, int verbose)
{
   struct stat perm;
   block_t *cur = head->head;
   block_t *next;

   /* testing filename again */
   if(!name) {
      if(verbose > 3) {	    
		   fprintf(stderr, "[ERROR] arg for Logs\n");
		   fprintf(stderr, "[ERROR] DEBUG name=%s\n",name );
      }
	   return -1;
   }

   /* do not clean files so often */
   while(cur) {
	   next = cur->next;
	   if(strcmp(name, cur->Log->name) == 0) {
	      if(verbose > 5) {
		      printf("Is OK\n");
	      }
	      return -2;
	   }
	   cur = next;
   }

   /* does this file exist? */
   if(stat(name, &perm) != 0) {	/* file exits? */
	   if(verbose > 4) {
         printf("[error] no stat for %s   ->Don't exist => Ignore It\n", name);
	   }
	   return -3;
   }
   
   /* is it a normal, not empty  file? */
   if((!S_ISREG(perm.st_mode)) || (perm.st_size <= 0)) {
	   if(verbose > 4) {
         fprintf(stderr,"[ERROR] not a regular file or it's empty %s =>\
                 Ignore It\n", name);
	   }
	   return -4;
   }

   /* do we write access this file? */
   if(access(name, W_OK) < 0) {
	   if(verbose > 2) {
	      fprintf(stderr,"[WARN]\a %s Don't have write permission!\
                 (Get root and fix it!) \t=> Ignore It Now\n", name);
	   }
	   return -5;
   }
   return 0;
}

/*
 * adding to the list
 */
int addLog(block_head_t * head, struct Cleanme *MyLog)
{
   int len = 0;
   block_t *cur;
   block_t *next;

   /* dummy checks */
   len = strlen(MyLog->name);
   
   if(len < 2 || len > BUFF_LONG - 1) {
      fprintf(stderr, "[ERROR] arg for Logs (len < 2 )%s \n", MyLog->name);
	   return -1;
   }
   len = strlen(MyLog->searchstring);
    
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs (MyLog->searchstring > BUFF_SHORT - 1) %s\n", MyLog->searchstring);
	   return -1;
   }
   len = strlen(MyLog->replacestring);
   
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs (MyLog->replacestring > BUFF_SHORT - 1) %s\n",MyLog->replacestring);
	   return -1;
   }
   len = strlen(MyLog->user);
   
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs (MyLog->user > BUFF_SHORT - 1) %s\n",MyLog->user);
	   return -1;
   }
   len = strlen(MyLog->tty);
   
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs (MyLog->tty > BUFF_SHORT - 1) %s\n",MyLog->tty);
	   return -1;
   }
   len = strlen(MyLog->cmd);
   
   if(len > BUFF_SHORT - 1) {
      fprintf(stderr, "[ERROR] arg for Logs (MyLog->cmd> BUFF_SHORT - 1) %s\n",MyLog->cmd);
	   return -1;
   }
   len = strlen(MyLog->logto);
    
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs (MyLog->logto > BUFF_SHORT - 1) %s\n",MyLog->logto);
	   return -1;
   }
   len = strlen(MyLog->crypt);
   
   if(len > BUFF_SHORT - 1) {
	   fprintf(stderr, "[ERROR] arg for Logs  (MyLog->crypt > BUFF_SHORT - 1) %s\n",MyLog->crypt);
	   return -1;
   }
   
   if(checkLog(head, MyLog->name, MyLog->verbose) != 0) {
	   if(MyLog->verbose > 4) {
	      fprintf(stderr, "[ERROR] cannot clean %s\n", MyLog->name);
	   }
	   return -2;
   }

   listAppend(head);
   cur = head->head;

   while(cur) {
	   next = cur->next;
	   if(cur->Log->logtype == 0) {
	      strncpy(cur->Log->name, MyLog->name, BUFF_LONG - 1);
	      cur->Log->name[BUFF_LONG - 1] = 0x0;
	      strncpy(cur->Log->searchstring, MyLog->searchstring,
		   BUFF_SHORT - 1);
	      cur->Log->searchstring[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->replacestring, MyLog->replacestring,
		   BUFF_SHORT - 1);
	      cur->Log->replacestring[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->user, MyLog->user, BUFF_SHORT - 1);
	      cur->Log->user[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->tty, MyLog->tty, BUFF_SHORT - 1);
	      cur->Log->tty[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->cmd, MyLog->cmd, BUFF_SHORT - 1);
	      cur->Log->cmd[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->logto, MyLog->logto, BUFF_SHORT - 1);
	      cur->Log->logto[BUFF_SHORT - 1] = 0x0;
	      strncpy(cur->Log->crypt, MyLog->crypt, BUFF_SHORT - 1);
	      cur->Log->crypt[BUFF_SHORT - 1] = 0x0;
	      cur->Log->logtype = MyLog->logtype;
	      cur->Log->compressed = MyLog->compressed;
	      cur->Log->verbose = MyLog->verbose;
	      cur->Log->is_clean = MyLog->is_clean;
	      cur->Log->checked = MyLog->checked;
	      cur->Log->was_clean = MyLog->was_clean;
	      cur->Log->ignore = MyLog->ignore;
	      cur->Log->change_time = MyLog->change_time;
	      cur->Log->delete_time = MyLog->delete_time;
	      cur->Log->from_time = MyLog->from_time;
	      cur->Log->to_time = MyLog->to_time;
	}
	cur = next;
 }
   return 0;
}
