#include "../include/config.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

/* own */
#include <logcl.h>
#include <misc_listen.h>
#include <misc_add.h>
#include <misc_dirs.h>

int checkdir(char *file)
{
   struct stat perm;
   
   if(stat(file, &perm) != 0) {
      return -1;
   }
   
   /* file exits? */
   if(S_ISDIR(perm.st_mode)) {
      return 0;
   } else
      return 1;
}

int scanDirs(block_head_t * mylog, struct Cleanme *MyLog)
{
   DIR *directory;
   struct dirent *dir;
   struct stat perm;
   char fullpath[BUFF_LONG + 1] = { "\0" };
   char file[BUFF_LONG + 1] = { "\0" };
   int compressed=0;

   if(MyLog->verbose > 5) {
      printf("[VERBOSE] scanning %s (logtype=%d)\n", MyLog->name,
             MyLog->logtype);
   }

   if(stat(MyLog->name, &perm) != 0) {	/* file exits? */
      if(MyLog->verbose > 6) {
         if (MyLog->verbose > 5){
            printf("[ERROR] no stat for %s\n", MyLog->name);
         }
	      return -1;
	   }
   }

   strncpy(file, MyLog->name, BUFF_LONG);
   MyLog->name[BUFF_LONG - 1] = 0x0;

   if(S_ISDIR(perm.st_mode)) {
	   if((directory = opendir(file))) {
         while ((dir = readdir(directory))) {
		      memset(fullpath, 0, BUFF_LONG);
		      if((strlen(file) + strlen(dir->d_name) + 2) < BUFF_LONG) {
		         if ((!strcmp(".", dir->d_name))
                   || (!strcmp("..", dir->d_name))) {
			      continue;
		      }
		      snprintf(fullpath, BUFF_LONG, "%s/%s", file,
			   dir->d_name);
		      memset(MyLog->name, 0, BUFF_LONG);
		      strncpy(MyLog->name, fullpath, BUFF_LONG - 1);
		      if(checkdir(fullpath) == 0) {
			      scanDirs(mylog, MyLog);
		      } else {
               compressed = ignore_file(fullpath);
                  MyLog->compressed = compressed;
                  addLog(mylog, MyLog);
		      }
		   }
	   }
   }
 }
   return 0;
}

/* 
 * Compressed files are not supported at the moment, sorry.
 * Ignore them, to avoid errors.
 *
 * */
int ignore_file(const char *file){
   int len;
   len=strlen(file);
   
   /* .gz */   
   if ((file[len-1] == 'z') && (file[len-2] == 'g') && (file[len-3] == '.')){
      return 1;
   }
   /* .zip */
   if ((file[len-1]=='p')&&(file[len-2]=='i')&&(file[len-3]=='z')&&(file[len-4]=='.')){
      return 5;
   }
   /* .bzip2 */
   if ((file[len-1]=='2')&&(file[len-2]=='p')&&(file[len-3]=='i')&&(file[len-4]=='z')&&(file[len-5]=='b')&&(file[len-6]=='.') ){
      return 3;
   }
   /* .tgz */   
   if ((file[len-1]=='z')&&(file[len-2]=='g')&&(file[len-3]=='t')&&(file[len-4]=='.')){
      return 2;
   }
   /* .tar */   
   if ((file[len-1]=='r')&&(file[len-2]=='a')&&(file[len-3]=='t')&&(file[len-4]=='.')){
      return 4;
   }
   /* .rar */
   if ((file[len-1]=='r')&&(file[len-2]=='a')&&(file[len-3]=='r')&&(file[len-4]=='.')){
      return 6;
   }
   /* the rest */
   return 0;
}
